/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */
 
package estella.cliente;
import  estella.lib.*;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.StringTokenizer;

/**
* Funcoes do cliente
* @author Cesar Kallas
* @since 21/11/2005
*/
public class Cliente implements Runnable
{
	private Constantes CON;
	private Log LOG;
  private Tempo TEMPO;
  private StringTokenizer servents;
	private Payload payload;
	private Descritor descritor;
  private Dados dados;
	
	public Cliente()
	{
    LOG = new Log("Cliente");
    CON = new Constantes();
    TEMPO = new Tempo();
    dados = new Dados();
		servents = new StringTokenizer(carregaServents(CON.LISTA_SERVENTS), ":");
	}
  
  public void run()
  {
		CON = new Constantes();
		conecta();
    LOG.fecha();
  }
  
	/**
	* Tenta conecta aos servents declarados na lista de servents pre-definidas em um arquivo
	* @return true ou false
	*/
  private boolean conecta()
  {
    String host;
    int porta;
    host = new String();
    
    // Tenta conectar, verifica resposta do servidor e se ok, adiciona a lista de conexoes
    try {
      while(servents.hasMoreTokens()) {
        host = servents.nextToken();
        porta = Integer.parseInt(servents.nextToken());
        Conexao conexao = new Conexao(host, porta);
        if(conexao.inicia()) {
          conexao.setTimeout(CON.SOCKET_TIMEOUT);
          conexao.envia(CON.GNUTELLA + "\n\n");
          if(conexao.recebe().indexOf(CON.GNUTELLA_OK) != -1) {
            dados.servents.adiciona(conexao);
            LOG.info("Conectado ao servent " + conexao.status());
						payload = new Payload();
						descritor = new Descritor(CON.PONG, CON.TTL, payload.pong(6346, "127.0.0.1", 1000, 55550));
						for(int i=0; i<3; i++)
							conexao.envia(new String(descritor.retorna()) + "\n");
						System.out.println("Tamanho do descritor pong: " + descritor.retorna().length + " TTL: " + descritor.ttl());
          }
        }
      }
    }
    catch(Exception FalhaConectar) {
      LOG.falha(FalhaConectar.getMessage());
    }
    if(dados.servents.tamanho() > 0)
      return true;
    else 
      return false;
  }
	
	/**
	* Retorna uma lista de servents pre-definidas em um arquivo
	* @param caminho_servents String com o caminho para o arquivo com os servents. Arquivo definido no formato --  'servent:porta' --
	* @return String com todos os servents e portas encontrados no arquivo
	*/
	public String carregaServents(String caminho_servents)
	{
		BufferedReader lista_servents;
		String servent, servents;
			
		try {
			lista_servents = new BufferedReader(new FileReader(new File(caminho_servents)));
			servents = new String();
			servent = new String();

			while(servent != null) {
				servent = lista_servents.readLine();
				if(servent == null)
					break;
				servents += servent + ":";
			}

			LOG.info("Servents carregados..." + caminho_servents);
			lista_servents.close();
			return servents;
		}
		catch(Exception LerServents) {
			LOG.falha(LerServents.getMessage());
			return null;
		}
	}
}
