/*
 * Copyright (C) 2000-2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 */
 
package estella.lib;
import  estella.lib.*;

import  java.io.File;
import  java.lang.String;

/**
* Gerencia repositorio de arquivos
* @author Cesar Kallas
* @since 05/12/2005
*/
public class Repositorio
{
	private static String caminho;
	private Arquivo arquivo;
	private Log LOG;
	private Constantes CON;
		
	public Repositorio()
	{
		caminho = CON.REPOSITORIO;
		LOG = new Log();
		gera();
	}
	
	/**
	* Gera repositorio
	*/
	public boolean gera()
	{
		try {
			File repositorio = new File(caminho);
			if(repositorio.isDirectory()) {
				geraRepositorio(new File(caminho));
				return true;
			}
			else
				return false;
		}
		catch(Exception SemRepositorio) {
			LOG.falha(SemRepositorio.getMessage());
			return false;
		}
	}
	
	/**
	* Lista todos arquivos (e arquivos dentro de subdiretorios) de um caminho passado
	* @param caminho diretorio inicial para busca de arquivos
	* @param Lista de arquivos definida na classe dados (Dados.arquivos)
	* @return true ou false
	* @see Dados
	*/
	private boolean geraRepositorio(File dir)
	{
		String[] arquivos = dir.list();
		try {
			for (int i = 0; i < arquivos.length; i++) {
				
				/** 
				* Se for um diretorio, entre nele e tente listar os arquivos (recursivamente)
				* Se nao, adiciona esse arquivo na lista
				*/
				File arquivoAtual = new File(dir, arquivos[i]);
				if (arquivoAtual.isDirectory())
					geraRepositorio(arquivoAtual);
				else {
				// Encontrado um novo arquivo, adiciona ele na lista de arquivos
					arquivo = new Arquivo();
					arquivo.nome = arquivos[i];
					arquivo.tamanho = (int)arquivoAtual.length();
					arquivo.index = System.identityHashCode(arquivoAtual);
					arquivo.caminho = dir.getName();
					Dados.arquivos.adiciona(arquivo);
					//LOG.info( arquivo.caminho + "\\" + arquivo.nome + "\t" + arquivo.tamanho + " index: " + arquivo.index);
				}
			}
			return true;
		}
		catch(Exception FalhaListarArquivos) {
			LOG.falha(FalhaListarArquivos.getMessage());
			return false;
		}
	}
}
